@@ -1,3 +1,26 @@
+/*
+ *      Copyright (C) 2000 Nikos Mavroyanopoulos
+ *
+ * This file is part of GNUTLS.
+ *
+ * GNUTLS is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * GNUTLS is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ */
+
+#ifndef DEFINES_H
+# define DEFINES_H
+
 #include <config.h>
 
 #ifdef STDC_HEADERS
@@ -22,6 +45,10 @@
 # include <sys/types.h>
 #endif
 
+#ifdef USE_GCRYPT
+# include <gcrypt.h>
+#endif
+
 #ifdef HAVE_SYS_STAT_H
 # include <sys/stat.h>
 #endif
@@ -79,3 +106,5 @@
 #  error "Neither memmove nor bcopy exists on your system."
 # endif
 #endif
+
+#endif /* defines_h */
