@@ -23,20 +23,30 @@
 #include "gnutls_algorithms.h"
 #include "gnutls_errors.h"
 
+/* include all the kx handler's definitions */
+#include "auth_anon.h"
+#include "auth_dhe_dss.h"
+#include "auth_srp.h"
+
+#define MAX_CIPHER 256
+#define MAX_MAC 256
+#define MAX_KX 256
+#define MAX_CIPHERSUITE 256
+#define MAX_COMPRESSION 256
+
 /* TLS Versions */
 
 typedef struct {
 	char *name;
-	GNUTLS_Version *id;
-	int supported;		/* 0 not supported, > 0 is supported */
+	GNUTLS_Version id; /* gnutls internal version number */
+	int major; /* defined by the protocol */
+	int minor; /* defined by the protocol */
+	int supported;	/* 0 not supported, > 0 is supported */
 } gnutls_version_entry;
 
-GNUTLS_Version GNUTLS_TLS1 = { 0, 3, 1 };
-GNUTLS_Version GNUTLS_SSL3 = { 0, 3, 0 };
-
 static gnutls_version_entry sup_versions[] = {
-	{"SSL3", &GNUTLS_SSL3, 1},
-	{"TLS1", &GNUTLS_TLS1, 1},
+	{"SSL3", GNUTLS_SSL3, 3, 0, 1},
+	{"TLS1", GNUTLS_TLS1, 3, 1, 1},
 	{0}
 };
 
@@ -45,7 +55,7 @@ static gnutls_version_entry sup_versions[] = {
                 for(p = sup_versions; p->name != NULL; p++) { b ; }
 
 #define GNUTLS_VERSION_ALG_LOOP(a) \
-                        GNUTLS_VERSION_LOOP( if( (p->id->local == version.local)&&(p->id->major == version.major)&&(p->id->minor == version.minor) ) { a; break; } )
+                        GNUTLS_VERSION_LOOP( if(p->id == version) { a; break; })
 
 
 #define GNUTLS_CIPHER_ENTRY(name, blksize, keysize, block, iv) \
@@ -64,6 +74,7 @@ typedef struct gnutls_cipher_entry gnutls_cipher_entry;
 static gnutls_cipher_entry algorithms[] = {
 	GNUTLS_CIPHER_ENTRY(GNUTLS_3DES, 8, 24, 1, 8),
 	GNUTLS_CIPHER_ENTRY(GNUTLS_RIJNDAEL, 16, 16, 1, 16),
+	GNUTLS_CIPHER_ENTRY(GNUTLS_RIJNDAEL256, 16, 32, 1, 16),
 	GNUTLS_CIPHER_ENTRY(GNUTLS_TWOFISH, 16, 16, 1, 16),
 	GNUTLS_CIPHER_ENTRY(GNUTLS_ARCFOUR, 1, 16, 0, 0),
 	GNUTLS_CIPHER_ENTRY(GNUTLS_NULL_CIPHER, 1, 0, 0, 0),
@@ -129,27 +140,28 @@ static gnutls_compression_entry compression_algorithms[] = {
 
 
 /* Key Exchange Section */
-#define GNUTLS_KX_ALGO_ENTRY(name, server_cert, server_kx, client_cert, RSA_premaster, DH_public_value) \
-	{ #name, name, server_cert, server_kx, client_cert, RSA_premaster, DH_public_value }
+#define GNUTLS_KX_ALGO_ENTRY(name, server_cert, client_cert, RSA_premaster, DH_public_value, auth_struct) \
+	{ #name, name, server_cert, client_cert, RSA_premaster, DH_public_value, auth_struct }
 
 struct gnutls_kx_algo_entry {
 	char *name;
 	KXAlgorithm algorithm;
 	int server_cert;
-	int server_kx;
 	int client_cert;
 	int RSA_premaster;
 	int DH_public_value;
+	MOD_AUTH_STRUCT *auth_struct;
 };
 typedef struct gnutls_kx_algo_entry gnutls_kx_algo_entry;
 
 static gnutls_kx_algo_entry kx_algorithms[] = {
-	GNUTLS_KX_ALGO_ENTRY(GNUTLS_KX_ANON_DH, 0, 1, 0, 0, 1),
-	GNUTLS_KX_ALGO_ENTRY(GNUTLS_KX_RSA, 1, 0, 1, 1, 0),
-	GNUTLS_KX_ALGO_ENTRY(GNUTLS_KX_DHE_DSS, 1, 1, 1, 0, 0),
-	GNUTLS_KX_ALGO_ENTRY(GNUTLS_KX_DHE_RSA, 1, 1, 1, 0, 0),
-	GNUTLS_KX_ALGO_ENTRY(GNUTLS_KX_DH_DSS, 1, 0, 1, 0, 0),
-	GNUTLS_KX_ALGO_ENTRY(GNUTLS_KX_DH_RSA, 1, 0, 1, 0, 0),
+	GNUTLS_KX_ALGO_ENTRY(GNUTLS_KX_ANON_DH, 0, 0, 0, 1, &anon_auth_struct),
+	GNUTLS_KX_ALGO_ENTRY(GNUTLS_KX_RSA,     1, 1, 1, 0, NULL),
+	GNUTLS_KX_ALGO_ENTRY(GNUTLS_KX_DHE_DSS, 1, 1, 0, 0, &dhe_dss_auth_struct),
+	GNUTLS_KX_ALGO_ENTRY(GNUTLS_KX_DHE_RSA, 1, 1, 0, 0, NULL),
+	GNUTLS_KX_ALGO_ENTRY(GNUTLS_KX_DH_DSS,  1, 1, 0, 0, NULL),
+	GNUTLS_KX_ALGO_ENTRY(GNUTLS_KX_DH_RSA,  1, 1, 0, 0, NULL),
+	GNUTLS_KX_ALGO_ENTRY(GNUTLS_KX_SRP,     0, 0, 0, 0, &srp_auth_struct),
 	{0}
 };
 
@@ -177,8 +189,15 @@ typedef struct {
 #define GNUTLS_DH_anon_3DES_EDE_CBC_SHA { 0x00, 0x1B }
 #define GNUTLS_DH_anon_ARCFOUR_MD5 { 0x00, 0x18 }
 #define GNUTLS_DH_anon_RIJNDAEL_128_CBC_SHA { 0x00, 0x34 }
+#define GNUTLS_DH_anon_RIJNDAEL_256_CBC_SHA { 0x00, 0x3A }
 #define GNUTLS_DH_anon_TWOFISH_128_CBC_SHA { 0xFF, 0x50 }
 
+#define GNUTLS_SRP_3DES_EDE_CBC_SHA { 0xF5, 0x1B }
+#define GNUTLS_SRP_ARCFOUR_MD5 { 0xF5, 0x18 }
+#define GNUTLS_SRP_RIJNDAEL_128_CBC_SHA { 0xF5, 0x34 }
+#define GNUTLS_SRP_RIJNDAEL_256_CBC_SHA { 0xF5, 0x3A }
+#define GNUTLS_SRP_TWOFISH_128_CBC_SHA { 0xF5, 0x50 }
+
 #define GNUTLS_DH_DSS_3DES_EDE_CBC_SHA { 0x00, 0x0D }
 #define GNUTLS_DH_RSA_3DES_EDE_CBC_SHA { 0x00, 0x10 }
 #define GNUTLS_DHE_DSS_3DES_EDE_CBC_SHA { 0x00, 0x13 }
@@ -198,14 +217,49 @@ typedef struct {
 #define GNUTLS_DHE_DSS_RIJNDAEL_128_CBC_SHA { 0x00, 0x32 }
 #define GNUTLS_DHE_RSA_RIJNDAEL_128_CBC_SHA { 0x00, 0x33 }
 
+#define GNUTLS_RSA_RIJNDAEL_256_CBC_SHA { 0x00, 0x35 }
+#define GNUTLS_DH_DSS_RIJNDAEL_256_CBC_SHA { 0x00, 0x36 }
+#define GNUTLS_DH_RSA_RIJNDAEL_256_CBC_SHA { 0x00, 0x37 }
+#define GNUTLS_DHE_DSS_RIJNDAEL_256_CBC_SHA { 0x00, 0x38 }
+#define GNUTLS_DHE_RSA_RIJNDAEL_256_CBC_SHA { 0x00, 0x39 }
+
 
 static gnutls_cipher_suite_entry cs_algorithms[] = {
+	/* anon */
 	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DH_anon_ARCFOUR_MD5,
 				  GNUTLS_ARCFOUR,
 				  GNUTLS_KX_ANON_DH, GNUTLS_MAC_MD5),
 	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DH_anon_3DES_EDE_CBC_SHA,
 				  GNUTLS_3DES, GNUTLS_KX_ANON_DH,
 				  GNUTLS_MAC_SHA),
+	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DH_anon_RIJNDAEL_128_CBC_SHA,
+				  GNUTLS_RIJNDAEL, GNUTLS_KX_ANON_DH,
+				  GNUTLS_MAC_SHA),
+	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DH_anon_RIJNDAEL_256_CBC_SHA,
+				  GNUTLS_RIJNDAEL256, GNUTLS_KX_ANON_DH,
+				  GNUTLS_MAC_SHA),
+	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DH_anon_TWOFISH_128_CBC_SHA,
+				  GNUTLS_TWOFISH, GNUTLS_KX_ANON_DH,
+				  GNUTLS_MAC_SHA),
+
+	/* SRP */
+	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_SRP_ARCFOUR_MD5,
+				  GNUTLS_ARCFOUR,
+				  GNUTLS_KX_SRP, GNUTLS_MAC_MD5),
+	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_SRP_3DES_EDE_CBC_SHA,
+				  GNUTLS_3DES, GNUTLS_KX_SRP,
+				  GNUTLS_MAC_SHA),
+	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_SRP_RIJNDAEL_128_CBC_SHA,
+				  GNUTLS_RIJNDAEL, GNUTLS_KX_SRP,
+				  GNUTLS_MAC_SHA),
+	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_SRP_RIJNDAEL_256_CBC_SHA,
+				  GNUTLS_RIJNDAEL256, GNUTLS_KX_SRP,
+				  GNUTLS_MAC_SHA),
+	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_SRP_TWOFISH_128_CBC_SHA,
+				  GNUTLS_TWOFISH, GNUTLS_KX_SRP,
+				  GNUTLS_MAC_SHA),
+
+	/* other! */
 	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DH_DSS_3DES_EDE_CBC_SHA,
 				  GNUTLS_3DES,
 				  GNUTLS_KX_DH_DSS, GNUTLS_MAC_SHA),
@@ -242,11 +296,20 @@ static gnutls_cipher_suite_entry cs_algorithms[] = {
 	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DHE_RSA_RIJNDAEL_128_CBC_SHA,
 				  GNUTLS_RIJNDAEL, GNUTLS_KX_DHE_RSA,
 				  GNUTLS_MAC_SHA),
-	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DH_anon_RIJNDAEL_128_CBC_SHA,
-				  GNUTLS_RIJNDAEL, GNUTLS_KX_ANON_DH,
+	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_RSA_RIJNDAEL_256_CBC_SHA,
+				  GNUTLS_RIJNDAEL256, GNUTLS_KX_RSA,
 				  GNUTLS_MAC_SHA),
-	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DH_anon_TWOFISH_128_CBC_SHA,
-				  GNUTLS_TWOFISH, GNUTLS_KX_ANON_DH,
+	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DH_DSS_RIJNDAEL_256_CBC_SHA,
+				  GNUTLS_RIJNDAEL256, GNUTLS_KX_DH_DSS,
+				  GNUTLS_MAC_SHA),
+	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DH_RSA_RIJNDAEL_256_CBC_SHA,
+				  GNUTLS_RIJNDAEL256, GNUTLS_KX_DH_RSA,
+				  GNUTLS_MAC_SHA),
+	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DHE_DSS_RIJNDAEL_256_CBC_SHA,
+				  GNUTLS_RIJNDAEL256, GNUTLS_KX_DHE_DSS,
+				  GNUTLS_MAC_SHA),
+	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DHE_RSA_RIJNDAEL_256_CBC_SHA,
+				  GNUTLS_RIJNDAEL256, GNUTLS_KX_DHE_RSA,
 				  GNUTLS_MAC_SHA),
 	{0}
 };
@@ -319,7 +382,7 @@ char *_gnutls_mac_get_name(MACAlgorithm algorithm)
 int _gnutls_mac_count()
 {
 	uint8 i, counter = 0;
-	for (i = 0; i < 256; i++) {
+	for (i = 0; i < MAX_MAC; i++) {
 		if (_gnutls_mac_is_ok(i) == 0)
 			counter++;
 	}
@@ -380,7 +443,7 @@ char *_gnutls_compression_get_name(CompressionMethod algorithm)
 int _gnutls_compression_count()
 {
 	uint8 i, counter = 0;
-	for (i = 0; i < 256; i++) {
+	for (i = 0; i < MAX_COMPRESSION; i++) {
 		if (_gnutls_compression_is_ok(i) == 0)
 			counter++;
 	}
@@ -476,7 +539,7 @@ char *_gnutls_cipher_get_name(BulkCipherAlgorithm algorithm)
 int _gnutls_cipher_count()
 {
 	uint8 i, counter = 0;
-	for (i = 0; i < 256; i++) {
+	for (i = 0; i < MAX_CIPHER; i++) {
 		if (_gnutls_cipher_is_ok(i) == 0)
 			counter++;
 	}
@@ -505,6 +568,14 @@ int _gnutls_kx_server_certificate(KXAlgorithm algorithm)
 
 }
 
+MOD_AUTH_STRUCT * _gnutls_kx_auth_struct(KXAlgorithm algorithm)
+{
+	MOD_AUTH_STRUCT * ret = NULL;
+	GNUTLS_KX_ALG_LOOP(ret = p->auth_struct);
+	return ret;
+
+}
+
 inline int _gnutls_kx_priority(GNUTLS_STATE state, KXAlgorithm algorithm)
 {
 	int i;
@@ -521,8 +592,88 @@ inline int _gnutls_kx_priority(GNUTLS_STATE state, KXAlgorithm algorithm)
 int _gnutls_kx_server_key_exchange(KXAlgorithm algorithm)
 {
 	size_t ret = 0;
+	void* ret2=NULL;
+	
+	/* if the auth algorithm does not have a null value 
+	 * for the kx2 generation then it supports it!
+	 */
+	GNUTLS_KX_ALG_LOOP(ret2 = p->auth_struct->gnutls_generate_server_kx);
+	if (ret2!=NULL) ret = 1;
 
-	GNUTLS_KX_ALG_LOOP(ret = p->server_kx);
+	return ret;
+
+}
+
+int _gnutls_kx_server_key_exchange2(KXAlgorithm algorithm)
+{
+	size_t ret = 0;
+	void* ret2=NULL;
+	
+	/* if the auth algorithm does not have a null value 
+	 * for the kx2 generation then it supports it!
+	 */
+	GNUTLS_KX_ALG_LOOP(ret2 = p->auth_struct->gnutls_generate_server_kx2);
+	if (ret2!=NULL) ret = 1;
+	
+	return ret;
+
+}
+
+int _gnutls_kx_client_key_exchange0(KXAlgorithm algorithm)
+{
+	size_t ret = 0;
+	void* ret2=NULL;
+	
+	/* if the auth algorithm does not have a null value 
+	 * for the kx0 generation then it supports it!
+	 */
+	GNUTLS_KX_ALG_LOOP(ret2 = p->auth_struct->gnutls_process_client_kx0);
+	if (ret2!=NULL) ret = 1;
+	
+	return ret;
+
+}
+int _gnutls_kx_client_key_exchange(KXAlgorithm algorithm)
+{
+	size_t ret = 0;
+	void* ret2=NULL;
+	
+	/* if the auth algorithm does not have a null value 
+	 * for the kx0 generation then it supports it!
+	 */
+	GNUTLS_KX_ALG_LOOP(ret2 = p->auth_struct->gnutls_process_client_kx);
+	if (ret2!=NULL) ret = 1;
+	return ret;
+
+}
+
+
+int _gnutls_kx_client_cert_vrfy(KXAlgorithm algorithm)
+{
+	size_t ret = 0;
+	void* ret2=NULL;
+	
+	/* if the auth algorithm does not have a null value 
+	 * for the cert_vrfy function then it supports it!
+	 */
+	GNUTLS_KX_ALG_LOOP(ret2 = p->auth_struct->gnutls_generate_client_cert_vrfy);
+	if (ret2!=NULL) ret = 1;
+	
+	return ret;
+
+}
+
+int _gnutls_kx_server_cert_vrfy(KXAlgorithm algorithm)
+{
+	size_t ret = 0;
+	void* ret2=NULL;
+	
+	/* if the auth algorithm does not have a null value 
+	 * for the cert_vrfy function then it supports it!
+	 */
+	GNUTLS_KX_ALG_LOOP(ret2 = p->auth_struct->gnutls_generate_server_cert_vrfy);
+	if (ret2!=NULL) ret = 1;
+	
 	return ret;
 
 }
@@ -575,7 +726,7 @@ char *_gnutls_kx_get_name(KXAlgorithm algorithm)
 int _gnutls_kx_count()
 {
 	uint8 i, counter = 0;
-	for (i = 0; i < 256; i++) {
+	for (i = 0; i < MAX_KX; i++) {
 		if (_gnutls_kx_is_ok(i) == 0)
 			counter++;
 	}
@@ -592,17 +743,33 @@ int _gnutls_kx_is_ok(KXAlgorithm algorithm)
 	else
 		ret = 1;
 	return ret;
+}
 
+int _gnutls_version_get_minor( GNUTLS_Version version) {
+int ret = -1;
+
+	GNUTLS_VERSION_ALG_LOOP(ret = p->minor);
+	return ret;
+}
+
+GNUTLS_Version _gnutls_version_get( int major, int minor) {
+int ret = -1;
+
+	GNUTLS_VERSION_LOOP(if ((p->major == major) && (p->minor==minor)) ret = p->id);
+	return ret;
+}
+
+int _gnutls_version_get_major( GNUTLS_Version version) {
+int ret = -1;
+
+	GNUTLS_VERSION_ALG_LOOP(ret = p->major);
+	return ret;
 }
 
 /* Version Functions */
 int _gnutls_version_cmp(GNUTLS_Version ver1, GNUTLS_Version ver2)
 {
-	if (ver1.major != ver2.major)
-		return 1;
-	if (ver1.minor != ver2.minor)
-		return 1;
-	if (ver1.local != ver2.local)
+	if (ver1 != ver2)
 		return 1;
 	return 0;
 }
@@ -690,11 +857,11 @@ int _gnutls_cipher_suite_count()
 	GNUTLS_CipherSuite suite;
 	int i, counter = 0, j;
 
-	for (j = 0; j < 256; j++) {
+	for (j = 0; j < MAX_CIPHERSUITE; j++) {
 		suite.CipherSuite[0] = j;
 		if (j!=0x00 && j!=0xFF) continue;
 
-		for (i = 0; i < 256; i++) {
+		for (i = 0; i < MAX_CIPHERSUITE; i++) {
 			suite.CipherSuite[1] = i;
 			if (_gnutls_cipher_suite_is_ok(suite) == 0)
 				counter++;
