@@ -116,3 +116,7 @@ void *_gnutls_get_kx_cred( GNUTLS_KEY key, int kx, int *err) {
 	if (err!=NULL) *err=0;
 	return ccred->credentials;
 }
+
+void* gnutls_get_auth_info( GNUTLS_STATE state) {
+	return &state->gnutls_key->auth_info;
+}
