@@ -53,38 +53,21 @@ const uint8 diffie_hellman_group1_prime[130] = { 0x04, 0x00,
 	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
 };
 
-int _gnutls_srp_gx(opaque *text, int textsize, opaque** result, opaque** ret_g, opaque** ret_n) {
+int _gnutls_srp_gn( opaque** ret_g, opaque** ret_n) {
 
-	MPI g, prime, x, e;
+	MPI g, prime;
 	size_t n = sizeof diffie_hellman_group1_prime;
-	int result_size, siz;
+	int siz;
 	char* tmp;
 
 	if (gcry_mpi_scan(&prime, GCRYMPI_FMT_USG,
 			  diffie_hellman_group1_prime, &n)) {
 		gnutls_assert();
-		return -1;
-	}
-	if (gcry_mpi_scan(&x, GCRYMPI_FMT_USG,
-			  text, &textsize)) {
-		gnutls_assert();
-		gcry_mpi_release(prime);
-		return -1;
+		return GNUTLS_E_MPI_SCAN_FAILED;
 	}
 
 	g = gcry_mpi_set_ui(NULL, SRP_G);
 
-	e = gcry_mpi_alloc_like(prime);
-
-	/* e = g^x mod prime (n) */
-	gcry_mpi_powm(e, g, x, prime);
-	gcry_mpi_release(x);
-
-	gcry_mpi_print(GCRYMPI_FMT_USG, NULL, &result_size, e);
-	if (result!=NULL) {
-		*result = gnutls_malloc(result_size);
-		gcry_mpi_print(GCRYMPI_FMT_USG, *result, &result_size, e);	
-	}
 
 	siz = 0;
 	gcry_mpi_print(GCRYMPI_FMT_USG, NULL, &siz, g);
@@ -92,7 +75,7 @@ int _gnutls_srp_gx(opaque *text, int textsize, opaque** result, opaque** ret_g,
 		tmp = gnutls_malloc(siz);
 		gcry_mpi_print(GCRYMPI_FMT_USG, tmp, &siz, g);
 
-		if (_gnutls_base64_encode( tmp, siz, ret_g) < 0)  {
+		if (_gnutls_sbase64_encode( tmp, siz, ret_g) < 0)  {
 			gnutls_free(tmp);
 			return GNUTLS_E_UNKNOWN_ERROR;
 		}
@@ -104,7 +87,7 @@ int _gnutls_srp_gx(opaque *text, int textsize, opaque** result, opaque** ret_g,
 	if (ret_n!=NULL) {
 		tmp = gnutls_malloc(siz);
 		gcry_mpi_print(GCRYMPI_FMT_USG, tmp, &siz, prime);
-		if (_gnutls_base64_encode( tmp, siz, ret_n) < 0) {
+		if (_gnutls_sbase64_encode( tmp, siz, ret_n) < 0) {
 			gnutls_free(tmp);
 			return GNUTLS_E_UNKNOWN_ERROR;
 		}
@@ -112,10 +95,40 @@ int _gnutls_srp_gx(opaque *text, int textsize, opaque** result, opaque** ret_g,
 		gnutls_free(tmp);
 	}
 	
-	gcry_mpi_release(e);
 	gcry_mpi_release(g);
 	gcry_mpi_release(prime);
 
+	return 0;
+
+}
+
+
+int _gnutls_srp_gx(opaque *text, int textsize, opaque** result, MPI g, MPI prime) {
+
+	MPI x, e;
+	int result_size;
+
+	if (gcry_mpi_scan(&x, GCRYMPI_FMT_USG,
+			  text, &textsize)) {
+		gnutls_assert();
+		return GNUTLS_E_MPI_SCAN_FAILED;
+	}
+
+	e = gcry_mpi_alloc_like(prime);
+
+	/* e = g^x mod prime (n) */
+	gcry_mpi_powm(e, g, x, prime);
+	gcry_mpi_release(x);
+
+	
+	gcry_mpi_print(GCRYMPI_FMT_USG, NULL, &result_size, e);
+	if (result!=NULL) {
+		*result = gnutls_malloc(result_size);
+		gcry_mpi_print(GCRYMPI_FMT_USG, *result, &result_size, e);	
+	}
+
+	gcry_mpi_release(e);
+
 	return result_size;
 
 }
@@ -225,33 +238,33 @@ int bits;
 /* generate x = SHA(s | SHA(U | ":" | p))
  * The output is exactly 20 bytes
  */
-void* _gnutls_calc_srp_sha( char* username, char* password, opaque* salt, int salt_size) {
+void* _gnutls_calc_srp_sha( char* username, char* password, opaque* salt, int salt_size, int* size) {
 GNUTLS_MAC_HANDLE td;
-opaque* hd;
+opaque* res;
 
+	*size = 20;
+	
 	td = gnutls_hash_init(GNUTLS_MAC_SHA);
 	gnutls_hash(td, username, strlen(username));
 	gnutls_hash(td, ":", 1);
 	gnutls_hash(td, password, strlen(password));
-	hd = gnutls_hash_deinit(td);
+	res = gnutls_hash_deinit(td);
 
 	td = gnutls_hash_init(GNUTLS_MAC_SHA);
 	gnutls_hash(td, salt, salt_size);
-	gnutls_hash(td, hd, 20);
-	gnutls_free(hd);
+	gnutls_hash(td, res, 20); /* 20 bytes is the output of sha1 */
+	gnutls_free(res);
 
-	hd = gnutls_hash_deinit(td);
-
-	return hd;	
+	return gnutls_hash_deinit(td);
 }
 
-void* _gnutls_calc_srp_x( char* username, char* password, opaque* salt, int salt_size, uint8 crypt_algo) {
+void* _gnutls_calc_srp_x( char* username, char* password, opaque* salt, int salt_size, uint8 crypt_algo, int* size) {
 	
 	switch(crypt_algo) {
 		case SRPSHA1_CRYPT:
-			return _gnutls_calc_srp_sha( username, password, salt, salt_size);
+			return _gnutls_calc_srp_sha( username, password, salt, salt_size, size);
 		case BLOWFISH_CRYPT:
-			return _gnutls_calc_srp_bcrypt( password, salt, salt_size);
+			return _gnutls_calc_srp_bcrypt( password, salt, salt_size, size);
 	}
 	return NULL;	
 }
@@ -269,6 +282,7 @@ MPI S, tmp1, tmp2, tmp4;
 	tmp2 = gcry_mpi_alloc_like(n);
 
 	gcry_mpi_powm(tmp1, g, x, n);
+
 	gcry_mpi_subm(tmp2, B, tmp1, n);
 
 	tmp4 = gcry_mpi_alloc_like(n);
